From b942c13395b07632020099a62a0121b81b395d98 Mon Sep 17 00:00:00 2001
From: hanishkvc <hanishkvc@gmail.com>
Date: Thu, 5 May 2022 01:47:52 +0530
Subject: [PATCH 3/5] Resolv:DnsRandNext: cprng?URandom+prng?Random to reduce
 overhead

As reading /dev/urandom (which needs to generate the next seq in a
potentially cprng? and cross kernel-userspace boundry) can take bit
more time than generating the next sequence in a prng on userspace
side (ie random c call). So switch to a hybrid of urandom+random,
where random gets reseeded periodically.
---
 libc/inet/resolv.c | 12 +++++++++---
 1 file changed, 9 insertions(+), 3 deletions(-)

diff --git a/libc/inet/resolv.c b/libc/inet/resolv.c
index f089aa0f1..7d952af28 100644
--- a/libc/inet/resolv.c
+++ b/libc/inet/resolv.c
@@ -1046,11 +1046,17 @@ static int __decode_answer(const unsigned char *message, /* packet */
 	return i + RRFIXEDSZ + a->rdlength;
 }
 
-
+#define DNSRAND_RESEED_CNT 128
 int dnsrand_next(int urand_fd, int def_value) {
+	static int cnt = -1;
 	if (urand_fd == -1) return def_value;
-	int val;
-	if(read(urand_fd, &val, sizeof(int)) != sizeof(int)) return def_value;
+	cnt += 1;
+	if ((cnt%DNSRAND_RESEED_CNT) == 0) {
+		int ival;
+		if(read(urand_fd, &ival, sizeof(int)) != sizeof(int)) return def_value;
+		srandom(ival);
+	}
+	int val = random(); // We dont need reproducible pseudo-random seq behaviour, so not bothering with random_r
 	return val;
 }
 
-- 
2.20.1

